#!/usr/bin/perl


# This script calculates Average Dynamic Recall and nDCG.
#
# Input: ground truth as HTML file (will be parsed by this script),
#        result list to be evaluated (on the command line).
#
# Author: Rainer Typke. Copyright 2005. All rights reserved.
#
# Parameters:
#  adr.pl ground_truth_html_file "comma-separated_list_of_matches"
#
# configure here whether the comma-separated matches are
#    RISM IDs (in the format xxx.xxx.xxx-x.x.x), $rism_ids=1,  or
#    numeric SQL database IDs, $rism_ids = 0:
$rism_ids = 1;
#



$gt = read_file($ARGV[0]);

$gt =~ /^.*?Ground truth for (.*?)\/(\d+).*?<table><tbody>(.*?)<\/tbody><\/table>(.*)$/gsi;

($forrism, $for, $gt) = ($1, $2, $3);

# print "threw away $4.\n";

print "ground truth for $forrism/$for:\n";

$erg = "";

# print $gt;

$gno = 0;

while ($gt =~ /^(.*?)bgcolor=..000000(.*?)<tr><td>(.*?)(<tr><td colspan="3" bgcolor="#000000">.*)$/si) {

  $gt = $4;
  # print "skipping $1 and $2, keeping $3 and ...\n\n";
  $gr = extract_group($rism_ids, $3);
  $group[$gno] = $gr;
  $erg .= "($gr)";
  $gno++;
}

if ($gt =~ /^(.*?)bgcolor=..000000(.*?)<tr><td>(.*?)$/si) { $gt = $3; }

$gr = extract_group($rism_ids, $gt);
$erg .= "($gr)";
$group[$gno] = $gr;

$groups = $gno + 1; 

print "$erg\n\n";


#@group = ("1,2,3","4,5","6,7,8");
#$groups = 3;
#@m = (1,4,3,2,5,6,7,8,9,9);
#@m = (1,2,3,4,9,9,9,9,9,9);



if ($ARGV[1] ne "") 
{
  $gtsize = 0;
  if ($ARGV[1] =~ /^.*\, .*$/gsi) {
    @m_raw = split(", ", $ARGV[1]); }
  else {
    @m_raw = split(",", $ARGV[1]);
  }

  foreach $m(@m_raw) {
    if (!$enc_raw{$m}) {
      push @m, $m;
    }
    $enc_raw{$m} = 1;
  }

  while ($#m < 99) { push @m, 0; }

  print "clean list: ".join(", ", @m)."\n";

  $gno = 0;
  $border = 0;
  $n = 1;
  $done = 0;
  foreach $m (@m) {
    if ($border < $n) {
      if ($gno < $groups) {
	if ($border > 0) {
	  $found = 0;
	  foreach $e (keys %encountered) {
	    if ($in_gt{$e}) {
	      $found++;
	    }
	  }
	  my $rec = $found / $border;
	  push @recall, $rec;
	  push @borders, $border;
#	  print "recall after $border: $rec\n";
	}
	@grp = split(",", $group[$gno]);
	foreach $g (@grp) {
	  $in_gt{$g} = 1;
	  $gtsize++;
	}
	$border += $#grp+1;
	$gno++;
      }
      else {
	$done = 1;
      }
    }
    if (!$done) {
      $encountered{$m} = 1; $n++;
      print "enc. $m ($border).  ";
    }
  }

  $found = 0;
  foreach $e (keys %encountered) {
    if ($in_gt{$e}) {
      $found++;
    }
  }
  my $rec = $found / $border;
  push @recall, $rec;
  push @borders, $border;
#  print "recall after $border: $rec\n";




  $meas = 0;
  $prevbord = 0;
  for ($n=0; $n<=$#recall; $n++) {
    $meas += ($borders[$n]-$prevbord)*$recall[$n];
    $prevbord = $borders[$n];
  }
  $meas /= $borders[$#borders];
#  print "\nResult quality: $meas\n";


  print "--------new measure-------
CSV: doc./found/recall\n";

  $gno = 0;
  $border = 0;
  $n = 1;
  $done = 0;
  $gtsize = 0;
  $warning = 0;

  $sum = 0;   # sum of all recalls (at each pos.)
  %in_gt = ();

  foreach $m (@m) {
    if ($border < $n) {
      if ($gno < $groups) {
	@grp = split(",", $group[$gno]);
	foreach $g (@grp) {
	  $in_gt{$g} = 1;
	  $gtsize++;
	}
	$border += $#grp+1;
	$gno++;
      }
      else {
	$done = 1;
      }
    }
    if (!$done) {
      push @topn, $m;
      $found = 0;
      if ($m == 0) { $warning = 1; }
      foreach $el (@topn) {
	if ($in_gt{$el}) {
	  $found++; ## print "$el is in gt\n";
	}
	##else { print "$el is not in gt\n";}
      }
      print "$n,$found,".($found/$n)."\n";
      $sum += $found/$n;
      $n++;
    }
  }
  $n--;
  $meas = $sum/$n;

  print "\nADR: $meas\n";

  if ($n < $gtsize) {
    print "\n\nERROR: $n documents, but ground truth has $gtsize documents!\n\n";
  } 
  if ($gno < $groups) {
    print "\n\nERROR: $groups groups, but we only looked at $gno groups!\n\n";
  }
  if ($warning) {
    print "\n\nWARNING:  RAN OUT OF ITEMS!!!\n\n";
  }


# nDCG:
  print "nDCG (normalized area): 2,1/r: ".nDCG(2,"1/r",@m).", 2,100-r: ".nDCG(2,"100-r",@m)."\n";


}



sub nDCG {
  my ($logbase, $scoremeth, @m) = @_;

  # compute pseudo relevance scores:
  my $n = 0;
  my $maxpos = 0;
  my %idealgain = ();
  my @idealcg;
  my @idealdcg;
  my @cg;
  my @dcg;
  my @ndcg;

  for ($g = 0; $g<$groups; $g++) {
    my $score;
    if ($scoremeth eq "1/r") {
      $score = 1/($n+1);
    } elsif ($scoremeth eq"100-r") {
      $score = 100-$n-1;
    } else {die "invalid score method $scoremeth."; }
    foreach $i (split(",",$group[$g])) {
      $idealgain{$i} = $score;
      $idealcg[$n] = $score;
      if ($n > 0) {
	$idealcg[$n] += $idealcg[$n-1];
      }
      if (($n+1) < $logbase) {
	$idealdcg[$n] = $idealcg[$n];
      } else {
	$idealdcg[$n] = log($idealcg[$n]) / log($logbase);
      }

#      print " ideal dcg[$n] = ".$idealdcg[$n]."\n";
      $n++; $maxpos++;
    }
  }

  @cg = ($idealgain{$m[0]}+0);
  @dcg = @cg;
  @ndcg = ( $dcg[0] / $idealdcg[0]);
  my $area = 1-$ndcg[0];
  for ($n = 1; $n < $maxpos; $n++) {
    push @cg, $idealgain{$m[$n]}+$cg[$n-1];
    if (($n+1) < $logbase) {
      push @dcg, $cg[$n];
    } else {
      if ($cg[$n] <= 0) {
	push @dcg, 0;
      } else {
	push @dcg, log($cg[$n]) / log($logbase);
      }
    }
    push @ndcg, $dcg[$n] / $idealdcg[$n];
    $area += 1-$ndcg[$n];
  }


  $n = 0;
  foreach $ndcg (@ndcg) {
#    print "cg[$n] = ".$cg[$n].", dcg[$n] = ".$dcg[$n].", ndcg[$n] = ".$ndcg[$n]."\n";
    $n++;
  }


  my $ntndcg = 1-$area / $maxpos;

#  print "normalized total nDCG area: ".($ntndcg)." (items: $maxpos)\n";
  return $ntndcg;
}


sub extract_group {
  my ($rism, $g) = @_;

#  print "\n\n$g\n";

  my @erg = (); my @rismerg = ();
  while ($g =~ /^(<tr><td>)?\s*(\d\d\d\.\d\d\d\.\d\d\d-\d+\.\d+\.\d+)\/(\d+):.*?<\/tr>(.*)$/si) {
    push @erg, $3; $g = $4;
    push @rismerg, $2;
  }
  if ($rism) {
    return join(",", @rismerg);
  }else {
    return join(",", @erg);
  }
}

#-------------------------------------------------------------
sub read_file {
  my ($fn) = @_;
  if (!(open FH, $fn))
    {
      return "";
    }
  my $res = "";
  while (<FH>) { $res .= $_; } # read all lines
  close (FH);
  return $res;
}
